﻿<?php
/**
 * Plugin Name: Google News Widget
 * Plugin URI: http://www.stefaniamarchisio.com/goggle-news-widget-shortcode-plugin/
 * Description: It shows news by Google given: localization, searchword(s) or topic and the number of news. It works as a widget or shortcode (in post/pages).
 * Version: 0.3
 * Author: Stefania Marchisio
 * Author URI: http://stefaniamarchisio.com/about
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */


/* 
This is a wordpress plugin compatible with wordpress 2.8+ as a widget; wordpress 2.5+ as shortcode.
From the readme.txt 
It shows news by Google given: localization, searchword(s) or topic and the number of news. It works as a widget or shortcode (in post/pages).


Copyright (C) 2010  Stefania Marchisio (email: mywizardwebs@gmail.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

/**
 * Add shortcode to widgets_init 
 * @since 0.2
 */
add_shortcode('gnews', 'GnewsShow');

/**
 * Add function to widgets_init
 * @since 0.1
 */
add_action( 'widgets_init', 'load_wp_googleNews' );

/**
 * Register our widget.
 * 'WP_GoogleNews' is the widget class used below.
 *
 * @since 0.1
 */
function load_wp_googleNews() {
   register_widget( 'WP_GoogleNews' );
}

/**
 * GoogleNews Widget class.
 * This class handles everything that needs to be handled with the widget:
 * the settings, form, display, and update.  Nice!
 *
 * @since 0.1
 */
class WP_GoogleNews extends WP_Widget {

	/**
	 * Widget setup.
	 */
	function WP_GoogleNews() {
		/* Widget settings. */
		$widget_ops = array( 'classname' => 'googlenews', 'description' => __('It provides a widget showing the latest Google News given (a) keyword(s) & location.', 'googlenews') );

		/* Widget control settings. */
		$control_ops = array( 'width' => 300, 'height' => 350, 'id_base' => 'googlenews-widget' );

		/* Create the widget. */
		$this->WP_Widget( 'googlenews-widget', __('Google News Widget', 'googlenews'), $widget_ops, $control_ops );
	}

	/**
	 * How to display the widget on the screen.
	 */
	function widget( $args, $instance ) {
		extract( $args );

		/* Our variables from the widget settings. */
		$title = apply_filters('widget_title', $instance['title'] );
		$name = $instance['name'];
		$topic = $instance['topic'];
		$location = $instance['location'];
		$number= $instance['number'];
		$list = $instance['list'];
		$target= $instance['target'];
		// $show_sex = isset( $instance['show_sex'] ) ? $instance['show_sex'] : false;

		/* Before widget (defined by themes). */
		echo $before_widget;

		/* Display the widget title if one was input (before and after defined by themes). */
		if ( $title )
			echo $before_title . $title . $after_title;

		/* Define the feed according to params entered */
		$feed = "http://news.google.com/news?cf=all&ned=" . $location. "&output=rss";
		if ( $name ) { $feed.= "&q=$name"; }
		if ( $topic ) { $feed.= "&topic=$topic"; }

		echo RSSParser($feed, $number, $list, $target);

		/* After widget (defined by themes). */
		echo $after_widget;
	}

	/**
	 * Update the widget settings.
	 */
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		/* Strip tags for title and name to remove HTML (important for text inputs). */
		$instance['title'] = strip_tags( $new_instance['title'] );
		$instance['name'] = strip_tags( $new_instance['name'] );
		$instance['number'] = $new_instance['number'];

		/* No need to strip tags for location (from drop-down menu) */
		$instance['location'] = $new_instance['location'];
		$instance['topic'] = $new_instance['topic'];

		$instance['target'] = $new_instance['target'];
		$instance['list'] = $new_instance['list'];

		return $instance;

	}

	/**
	 * Displays the widget settings controls on the widget panel.
	 * Make use of the get_field_id() and get_field_name() function
	 * when creating your form elements. This handles the confusing stuff.
	 */
	function form( $instance ) {

		/* Set up some default widget settings. */
		$defaults = array( 'title' => __('The Latest Google News', 'googlenews'), 'name' => __('sport', 'googlenews'), 'location' => 'uk', 'number' => '15');
		$instance = wp_parse_args( (array) $instance, $defaults ); ?>

		<!-- Widget Title: Text Input -->
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e('Widget Title:', 'hybrid'); ?></label>
			<input id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $instance['title']; ?>" style="width:100%;" />
		</p>

		<!-- Search Keyword(s): Text Input -->
		<p>
			<label for="<?php echo $this->get_field_id( 'name' ); ?>"><?php _e('Search keyword(s):', 'googlenews'); ?></label>
			<input id="<?php echo $this->get_field_id( 'name' ); ?>" name="<?php echo $this->get_field_name( 'name' ); ?>" value="<?php echo $instance['name']; ?>" style="width:100%;" />
		</p>
                <p align="center">=== OR ===</p>
		<!-- Topic: Select Box -->
		<p>
		<label for="<?php echo $this->get_field_id( 'topic' ); ?>"><?php _e('Topic:', 'googlenews'); ?></label> 
		<select id="<?php echo $this->get_field_id( 'topic' ); ?>" name="<?php echo $this->get_field_name( 'topic' ); ?>" class="widefat" style="width:100%;">
// options in Google News on 20 Feb 2010
<option value="" <?php if ('' == $instance['topic']) echo 'selected';?>>Top Stories (default)</option>
<option value="w" <?php if ('w' == $instance['topic']) echo 'selected';?>>World</option>
<option value="n" <?php if ('n' == $instance['topic']) echo 'selected';?>>Nation</option>
<option value="b" <?php if ('b' == $instance['topic']) echo 'selected';?>>Business</option>
<option value="t" <?php if ('t' == $instance['topic']) echo 'selected';?>>Sci/Tecn</option>
<option value="e" <?php if ('e' == $instance['topic']) echo 'selected';?>>Entertainment</option>
<option value="s" <?php if ('s' == $instance['topic']) echo 'selected';?>>Sports</option>
<option value="m" <?php if ('m' == $instance['topic']) echo 'selected';?>>Health</option>
<option value="ir" <?php if ('ir' == $instance['topic']) echo 'selected';?>>Spotlight</option>
<option value="po" <?php if ('po' == $instance['topic']) echo 'selected';?>>Most Popular</option>
		</select>
		</p>

		<!-- Location: Select Box -->
		<p>
		<label for="<?php echo $this->get_field_id( 'location' ); ?>"><?php _e('Location:', 'googlenews'); ?></label> 
		<select id="<?php echo $this->get_field_id( 'location' ); ?>" name="<?php echo $this->get_field_name( 'location' ); ?>" class="widefat" style="width:100%;">

// options in Google News on 22 Jan 2010
<option value="es_ar" <?php if ('es_ar' == $instance['location']) echo 'selected';?>>Argentina</option>
<option value="au" <?php if ('au' == $instance['location']) echo 'selected';?>>Australia</option> 
<option value="nl_be" <?php if ('nl_be' == $instance['location']) echo 'selected';?>>België</option> 
<option value="fr_be" <?php if ('fr_be' == $instance['location']) echo 'selected';?>>Belgique</option> 
<option value="en_bw" <?php if ('en_bw' == $instance['location']) echo 'selected';?>>Botswana</option> 
<option value="pt-BR_br" <?php if ('pt-BR_br' == $instance['location']) echo 'selected';?>>Brasil</option> 
<option value="ca" <?php if ('es_ar' == $instance['location']) echo 'selected';?>>Canada English</option> 
<option value="fr_ca" <?php if ('fr_ca' == $instance['location']) echo 'selected';?>>Canada Français</option> 
<option value="cs_cz" <?php if ('cs_cz' == $instance['location']) echo 'selected';?>>Česká republika</option> 
<option value="es_cl" <?php if ('es_cl' == $instance['location']) echo 'selected';?>>Chile</option> 
<option value="es_co" <?php if ('es_co' == $instance['location']) echo 'selected';?>>Colombia</option> 
<option value="es_cu" <?php if ('es_cu' == $instance['location']) echo 'selected';?>>Cuba</option> 
<option value="de" <?php if ('de' == $instance['location']) echo 'selected';?>>Deutschland</option> 
<option value="es" <?php if ('es' == $instance['location']) echo 'selected';?>>España</option> 
<option value="es_us" <?php if ('es_us' == $instance['location']) echo 'selected';?>>Estados Unidos</option> 
<option value="en_et" <?php if ('en_et' == $instance['location']) echo 'selected';?>>Ethiopia</option> 
<option value="fr" <?php if ('fr' == $instance['location']) echo 'selected';?>>France</option> 
<option value="en_gh" <?php if ('en_gh' == $instance['location']) echo 'selected';?>>Ghana</option> 
<option value="in" <?php if ('in' == $instance['location']) echo 'selected';?>>India</option> 
<option value="en_ie" <?php if ('en_ie' == $instance['location']) echo 'selected';?>>Ireland</option> 
<option value="en_il" <?php if ('en_il' == $instance['location']) echo 'selected';?>>Israel English</option> 
<option value="it" <?php if ('it' == $instance['location']) echo 'selected';?>>Italia</option> 
<option value="en_ke" <?php if ('en_ke' == $instance['location']) echo 'selected';?>>Kenya</option> 
<option value="hu_hu" <?php if ('hu_hu' == $instance['location']) echo 'selected';?>>Magyarország</option> 
<option value="en_my" <?php if ('en_my' == $instance['location']) echo 'selected';?>>Malaysia</option> 
<option value="es_mx" <?php if ('es_mx' == $instance['location']) echo 'selected';?>>México</option> 
<option value="en_na" <?php if ('es_na' == $instance['location']) echo 'selected';?>>Namibia</option> 
<option value="nl_nl" <?php if ('nl_nl' == $instance['location']) echo 'selected';?>>Nederland</option> 
<option value="nz" <?php if ('nz' == $instance['location']) echo 'selected';?>>New Zealand</option> 
<option value="en_ng" <?php if ('en_ng' == $instance['location']) echo 'selected';?>>Nigeria</option> 
<option value="no_no" <?php if ('no_no' == $instance['location']) echo 'selected';?>>Norge</option> 
<option value="de_at" <?php if ('de_at' == $instance['location']) echo 'selected';?>>Österreich</option> 
<option value="en_pk" <?php if ('es_pk' == $instance['location']) echo 'selected';?>>Pakistan</option> 
<option value="es_pe" <?php if ('es_pe' == $instance['location']) echo 'selected';?>>Perú</option> 
<option value="en_ph" <?php if ('en_ph' == $instance['location']) echo 'selected';?>>Philippines</option> 
<option value="pl_pl" <?php if ('pl_pl' == $instance['location']) echo 'selected';?>>Polska</option> 
<option value="pt-PT_pt" <?php if ('pt-PT_pt' == $instance['location']) echo 'selected';?>>Portugal</option> 
<option value="de_ch" <?php if ('de_ch' == $instance['location']) echo 'selected';?>>Schweiz</option> 
<option value="fr_sn" <?php if ('fr_sn' == $instance['location']) echo 'selected';?>>Sénégal</option> 
<option value="en_sg" <?php if ('en_sg' == $instance['location']) echo 'selected';?>>Singapore</option> 
<option value="en_za" <?php if ('en_za' == $instance['location']) echo 'selected';?>>South Africa</option> 
<option value="fr_ch" <?php if ('fr_ch' == $instance['location']) echo 'selected';?>>Suisse</option> 
<option value="sv_se" <?php if ('sv_se' == $instance['location']) echo 'selected';?>>Sverige</option> 
<option value="en_tz" <?php if ('en_tz' == $instance['location']) echo 'selected';?>>Tanzania</option> 
<option value="tr_tr" <?php if ('tr_tr' == $instance['location']) echo 'selected';?>>Türkiye</option> 
<option value="uk" <?php if ('uk' == $instance['location']) echo 'selected';?>>U.K.</option> 
<option value="us" <?php if ('us' == $instance['location']) echo 'selected';?>>U.S.</option> 
<option value="en_ug" <?php if ('en_ug' == $instance['location']) echo 'selected';?>>Uganda</option> 
<option value="es_ve" <?php if ('es_ve' == $instance['location']) echo 'selected';?>>Venezuela</option> 
<option value="vi_vn" <?php if ('vi_vn' == $instance['location']) echo 'selected';?>>Việt Nam (Vietnam)</option> 
<option value="en_zw" <?php if ('en_zw' == $instance['location']) echo 'selected';?>>Zimbabwe</option> 
<option value="el_gr" <?php if ('el_gr' == $instance['location']) echo 'selected';?>>Ελλάδα (Greece)</option> 
<option value="ru_ru" <?php if ('ru_ru' == $instance['location']) echo 'selected';?>>Россия (Russia)</option> 
<option value="ru_ua" <?php if ('ru_ua' == $instance['location']) echo 'selected';?>>Украина / русский (Ukraine)</option> 
<option value="uk_ua">Україна / українська (Ukraine)</option> 
<option value="iw_il">ישראל (Israel)</option> 
<option value="ar_ae">الإمارات (UAE)</option> 
<option value="ar_sa">السعودية (KSA)</option> 
<option value="ar_me">العالم العربي (Arabic)</option> 
<option value="ar_lb">لبنان (Lebanon)</option> 
<option value="ar_eg">مصر (Egypt)</option> 
<option value="hi_in">हिन्दी (India)</option> 
<option value="ta_in">தமிழ்(India)</option>
<option value="te_in">తెలుగు (India)</option> 
<option value="ml_in">മലയാളം (India)</option> 
<option value="kr">한국 (Korea)</option> 
<option value="cn">中国版 (China)</option> 
<option value="tw">台灣版 (Taiwan)</option> 
<option value="jp">日本 (Japan)</option> 
<option value="hk">香港版 (Hong Kong)</option>

		</select>
		</p>

		<!-- N. articles to display: Text Input -->
		<p>
			<label for="<?php echo $this->get_field_id( 'number' ); ?>"><?php _e('News to display (max 15):', 'googlenews'); ?></label>
			<input id="<?php echo $this->get_field_id( 'number' ); ?>" name="<?php echo $this->get_field_name( 'number' ); ?>" value="<?php echo $instance['number']; ?>" style="width:20%;" />
		</p>

		<!-- List Type: Select Box -->
		<p>
		<label for="<?php echo $this->get_field_id( 'list' ); ?>"><?php _e('List Type:', 'googlenews'); ?></label> 
		<select id="<?php echo $this->get_field_id( 'list' ); ?>" name="<?php echo $this->get_field_name( 'list' ); ?>" class="widefat" style="width:100%;">
<option value="UL" <?php if ('UL' == $instance['list']) echo 'selected';?>>Unordered (or Dotted) List (default)</option>
<option value="OL" <?php if ('OL' == $instance['list']) echo 'selected';?>>Ordered (or Numbered) List</option>
		</select>
		</p>

		<!-- Target: Select Box -->
		<p>
		<label for="<?php echo $this->get_field_id( 'target' ); ?>"><?php _e('Target:', 'googlenews'); ?></label> 
		<select id="<?php echo $this->get_field_id( 'target' ); ?>" name="<?php echo $this->get_field_name( 'target' ); ?>" class="widefat" style="width:100%;">
<option value="_blank" <?php if ('_blank' == $instance['target']) echo 'selected';?>>Open link in a new window (default)</option>
<option value="_self" <?php if ('_self' == $instance['target']) echo 'selected';?>>Open link in the same window</option>
		</select>
		</p>
	<?php
	}
}

function GnewsShow($atts) {
     extract(shortcode_atts(array(
     "search" => '',
     "topic" => '',
     "location" => '',
     "num" => '3',
     "list" => 'UL',
     "target" => '_blank'
     ), $atts));

     /* Display name from widget settings if one was input. */
     $feed = "http://news.google.com/news?cf=all&ned=" . $location. "&output=rss";
     if ( $search ) { $feed .= "&q=$search"; }
     if ( $topic ) { $feed .= "&topic=$topic"; }

    return RSSParser($feed, $num, $list, $target);
}

function RSSParser($url,$size,$list,$target) {
        // display the parsed articles as a unordered list
	$doc  = new DOMDocument();
	$doc->load($url);

	$channels = $doc->getElementsByTagName("channel");
	
	$page = "<$list>\n";

	foreach($channels as $channel) {
		$items = $channel->getElementsByTagName("item");
		$cntitems = 0;

		foreach($items as $item) {
			if ($size > 0 && $cntitems < $size) {

				$tnl = $item->getElementsByTagName("title");
				$tnl = $tnl->item(0);
				$title = $tnl->firstChild->data;

				$tnl = $item->getElementsByTagName("link");
				$tnl = $tnl->item(0);
				$link = $tnl->firstChild->data;

				$tnl = $item->getElementsByTagName("description");
				$tnl = $tnl->item(0);
				$description = $tnl->firstChild->data;

				$page .= "<li><a target='$target' href=\"$link\">$title</a></li>\n";			
				$cntitems += 1;
			} //$size > 0 && $cntitem < $size)
		} // foreach($items as $item)
	} // foreach($channels as $channel)
	$page .= "</".$list.">\n";
        // $page .= "<br>[debug on: <a target='_blank' href='$url'>$url</a>]\n";
  return $page;
}
?>